
/**********************************************************************************
 Persistence of Vision Ray Tracer Scene Description File
 File name   : Geomorph_mcr.pov
 Version     : 3.6
 Description : A macro for generating isosurface landscapes, including islands 
               and single rocks
               
 Date        : December 2004 - January 2005
 Author      : Thomas A. M. de Groot
 E-mail      : t.degroot@inter.nl.net
 
 Copyright (C) 2005. All rights reserved.
**********************************************************************************/

#include "rand.inc"
#include "functions.inc"

default {
  texture {
    pigment { rgb <1,0,0> }
  }
}

// Run the macro as stand alone. Else, declare Standalone as false before including and calling the macro in your scene.
#ifndef (Standalone)     #declare Standalone = true;          #end

#if (Standalone)
global_settings {
  assumed_gamma 1.0
  max_intersections 64
  max_trace_level 5
  noise_generator 2
}
#end


// -------- Terrain textures (by Christoph Hormann) --------
#declare T_Sand=
texture {
  pigment { color rgb <1.1, 0.7, 0.3> }
  finish { 
    specular 0.06 
    ambient <0.8, 0.9, 1.4>*0.1
  }
  normal {
    granite 0.3
    scale 0.1
  }
}

#declare T_Grass=
texture {
  pigment { color rgb <0.5, 1.15, 0.3> }
  finish {
    specular 0.1
    diffuse 0.3
    brilliance 1.6
    ambient <0.8, 0.9, 1.4>*0.03
  }
  normal {
    granite 0.5
    accuracy 0.01
    scale 0.12
  }
}

#declare T_Rock=
texture {
  pigment {
    agate
    color_map {
      [0.2 color rgb <0.55, 0.50, 0.50> ]
      [0.6 color rgb <0.75, 0.50, 0.60> ]
      [1.0 color rgb <0.70, 0.60, 0.60> ]
    }
    scale 0.2
    warp { turbulence 0.5 }
  }
  finish {
    specular 0.2
    diffuse 0.4
    ambient <0.8, 0.9, 1.4>*0.06
  }
  normal {
    granite 0.6
    scale 0.1
  }
}

//======================================================================================================================================
#macro Geomorph() // start of the Geomorph macro

// (0) Declared landscape parameters:
// patterns:
#ifndef (Noised)				#local Noised = false;							#end
#ifndef (Ridged)				#local Ridged = false;							#end
#ifndef (Wrinkled)			#local Wrinkled = true;							#end
#ifndef (Rippled)				#local Rippled = false;							#end
#ifndef (Waved)					#local Waved = false;								#end
#ifndef (Dented)				#local Dented = false;							#end
#ifndef (Sphered)				#local Sphered = false;							#end		// when true, generates an island or an isolated rock
#ifndef (Crackled)			#local Crackled = false;						#end

// warp:
#ifndef (Wrinkleswarp)	#local Wrinkleswarp = true;					#end
#ifndef (Rippleswarp)		#local Rippleswarp = false;					#end
#ifndef (Waveswarp)			#local Waveswarp = false;						#end
#ifndef (Dentswarp)			#local Dentswarp = false;						#end
#ifndef (Cracklewarp)		#local Cracklewarp = false;					#end

// gain:
#ifndef (Wrinklesgain)	#local Wrinklesgain = 0.5;					#end 
#ifndef (Ripplesgain)		#local Ripplesgain = 0.5;	  				#end
#ifndef (Wavesgain)			#local Wavesgain = 0.5;	  					#end 
#ifndef (Dentsgain)			#local Dentsgain = 0.5;	  					#end  
#ifndef (Spheregain)		#local Spheregain = 0.5;	  				#end 
#ifndef (Cracklegain)		#local Cracklegain = 1.0;	  				#end 

// warp turbulence:
#ifndef (Wrinklesturb)	#local Wrinklesturb = <0.5, 0.5, 0.5>;		#end
#ifndef (Ripplesturb)		#local Ripplesturb = <0.5, 0.5, 0.5>;			#end
#ifndef (Wavesturb)			#local Wavesturb = <0.5, 0.5, 0.5>;				#end
#ifndef (Dentsturb)			#local Dentsturb = <0.5, 0.5, 0.5>;				#end
#ifndef (Crackleturb)		#local Crackleturb = <0.5, 0.5, 0.5>;			#end

// scale:
#ifndef (Wrinklescale)	#local Wrinklescale = <0.5, 0.2, 0.5>;			#end
#ifndef (Ripplescale)		#local Ripplescale = <0.5, 0.2, 0.5>;				#end
#ifndef (Wavescale)			#local Wavescale = <0.5, 0.2, 0.5>;					#end
#ifndef (Dentscale)			#local Dentscale = <0.5, 0.2, 0.5>;					#end
#ifndef (Cracklescale)	#local Cracklescale = <0.5, 0.2, 0.5>;			#end
#ifndef (Cracksize)			#local Cracksize = 2;												#end

// angle:
#ifndef (Wrinklesangle)	#local Wrinklesangle = <0, 0, -20>; #end
#ifndef (Ripplesangle)	#local Ripplesangle = <0, 0, -20>;	#end
#ifndef (Wavesangle)		#local Wavesangle = <0, 0, -20>;		#end
#ifndef (Dentsangle)		#local Dentsangle = <0, 0, -20>;		#end
#ifndef (Crackleangle)	#local Crackleangle = <0, 0, 0>;		#end

// sphere:
#ifndef (SphereShearX)	#local SphereShearX = 0;						#end		// shear of the sphere function in the x direction
#ifndef (SphereShearZ)	#local SphereShearZ = 0;						#end		// shear of the sphere function in the z direction
#ifndef (Diameter)			#local Diameter = 1;								#end		// diameter of the sphere function

// noise and randomness:
#ifndef (Noise)					#local Noise = 0.4;									#end		// controller for the f_noise3d function
#ifndef (Stream)				#local Stream = 92322;							#end		// a random seed

// isosurface:
#ifndef (UpperBound)		#local UpperBound = 3;							#end		// Bounding box upper limit
#ifndef (LowerBound)		#local LowerBound = -2;							#end		// Bounding box lower limit
#ifndef (MaxGrad)				#local MaxGrad = 10;								#end		// max_gradient of isosurface (change according to need)
#ifndef (Xsize)					#local Xsize = 10;									#end		// increase size of landscape (not scale!) for X (i.e. the bounding box)
#ifndef (Zsize)					#local Zsize = 10;									#end		// increase size of landscape (not scale!) for Z (i.e. the bounding box)
#ifndef (Isoscale)			#local Isoscale = <1, 1, 1>;				#end		// scale of the isosurface
#ifndef (Isotrans)			#local Isotrans = <0, 0, 0>;				#end		// translate the isosurface

// texture:
#ifndef (Simpletex)			#local Simpletex = false;						#end		// just a choice between two nice textures
#ifndef (Slope)					#local Slope = true;								#end		// true = slope & altitude; false = gradient texture
#ifndef (SlopeValue)		#local SlopeValue = 0.6;						#end		// (0-1) higher values: more slope, less altitude, and vice versa

// internal default textures of the macro:
#ifndef (Landtex)
#declare Landtex =
#if (Simpletex)
  texture {
    pigment {
      #if (Slope)
        slope {<0.0, SlopeValue, 0.0> , 0.0 , 0.5
          altitude <0.0, 1-SlopeValue, 0.0> , LowerBound , UpperBound  // bounding box goes from LowerBound to UpperBound
        }
      #else
        gradient y
      #end
      colour_map {
        [0.00 rgb <0.1,0.4,0.1>]
        [0.25 rgb <0.5,0.4,0.3>]
        [0.50 rgb 0.50]
        [0.75 rgb 0.75]
        [1.00 rgb 0.90]
      }
    }
  }
#else
  texture {
    #if (Slope)
      slope {<0.0, SlopeValue, 0.0> , 0.0 , 0.5
        altitude <0.0, 1-SlopeValue, 0.0> , LowerBound , UpperBound  // bounding box goes from LowerBound to UpperBound
      }
    #else
      gradient y
    #end
    texture_map {
      [0.00 T_Rock]
      [0.25 T_Grass]
      [0.50 T_Sand]
      [0.75 T_Sand]
      [1.00 T_Rock]
    }
  }
#end
#end // end of ndef

//------------------------
// (1) Basic pattern functions:
#if (Wrinkleswarp)
  #local Wrinkles=function {pattern {wrinkles warp {turbulence Wrinklesturb octaves 3 omega 0.5 lambda 1} scale Wrinklescale }}
#else
  #local Wrinkles=function {pattern {wrinkles scale Wrinklescale }}
#end

#if (Rippleswarp)
  #local Ripples=function {pattern {ripples frequency 0.2 warp {turbulence Ripplesturb octaves 3 omega 0.5 lambda 1} scale Ripplescale }}
#else
  #local Ripples=function {pattern {ripples frequency 0.2 scale Ripplescale }}
#end

#if (Waveswarp)
  #local Waves=function {pattern {waves poly_wave 1 warp {turbulence Wavesturb octaves 3 omega 0.5 lambda 1} scale Wavescale }}
#else
  #local Waves=function {pattern {waves poly_wave 0.5 scale Wavescale }}
#end

#if (Dentswarp)
  #local Dents=function {pattern {dents warp {turbulence Dentsturb octaves 5 omega 0.5 lambda 2} scale Dentscale }}
#else
  #local Dents=function {pattern {dents scale Dentscale }}
#end

// the crackle function generates vertical faults and fault faces in the rocks. Crackleb is more random
#if (Cracklewarp)
  #local Crackle=function {pattern {crackle solid warp {turbulence Crackleturb octaves 5 omega 0.5 lambda 2} scale Cracklescale*Cracksize }}
#else
  #local Crackle=function {pattern {crackle solid scale Cracklescale*Cracksize }}
#end

//------------------------
// (2) 'landscape'functions:
#local Seed = seed(Stream);
#local RandX = rand(Seed)*1000;
#local RandY = rand(Seed)*1000;
#local RandZ = rand(Seed)*1000;
#local Hetero=function {f_hetero_mf(x+RandX,y-RandY,z+RandZ, 0.8, 2, 1, 0, 0.9, 2)*1 }// H,lacunarity,octaves,offset,gain,ng
#local Ridge=function {f_ridged_mf(x+RandX,y+RandY,z+RandZ, 0.8, 2, 1, 0, 0.9, 2)*1 }// H,lacunarity,octaves,offset,gain,ng
#local Sphere=function {f_sphere(x,y,z,Diameter)} // this defines the "island". Change x,y, or z to 0 for subtle changes

//------------------------
// (3) first, rotation around x:
#local Wrinklesxangle = Wrinklesangle.x;
#local Wrinklesyangle = Wrinklesangle.y;
#local Wrinkleszangle = Wrinklesangle.z;
#local Wrinklesrotx=function {Wrinkles(x, z*sin(radians(Wrinklesxangle)) + y*cos(radians(Wrinklesxangle)), z*cos(radians(Wrinklesxangle)) - y*sin(radians(Wrinklesxangle))) }

#local Ripplesxangle = Ripplesangle.x;
#local Ripplesyangle = Ripplesangle.y;
#local Rippleszangle = Ripplesangle.z;
#local Ripplesrotx=function {Ripples(x, z*sin(radians(Ripplesxangle)) + y*cos(radians(Ripplesxangle)), z*cos(radians(Ripplesxangle)) - y*sin(radians(Ripplesxangle))) }

#local Wavesxangle = Wavesangle.x;
#local Wavesyangle = Wavesangle.y;
#local Waveszangle = Wavesangle.z;
#local Wavesrotx=function {Waves(x, z*sin(radians(Wavesxangle)) + y*cos(radians(Wavesxangle)), z*cos(radians(Wavesxangle)) - y*sin(radians(Wavesxangle))) }

#local Dentsxangle = Dentsangle.x;
#local Dentsyangle = Dentsangle.y;
#local Dentszangle = Dentsangle.z;
#local Dentsrotx=function {Dents(x, z*sin(radians(Dentsxangle)) + y*cos(radians(Dentsxangle)), z*cos(radians(Dentsxangle)) - y*sin(radians(Dentsxangle))) }

#local Cracklexangle = Crackleangle.x;
#local Crackleyangle = Crackleangle.y;
#local Cracklezangle = Crackleangle.z;
#local Cracklerotx=function {Crackle(x, z*sin(radians(Cracklexangle)) + y*cos(radians(Cracklexangle)), z*cos(radians(Cracklexangle)) - y*sin(radians(Cracklexangle))) }

//------------------------
// (4) then, rotation around z:
#local Wrinklesrotz=function {Wrinklesrotx(x*cos(radians(Wrinkleszangle)) + y*sin(radians(Wrinkleszangle)), -x*sin(radians(Wrinkleszangle)) + y*cos(radians(Wrinkleszangle)), z) }
#local Ripplesrotz=function {Ripplesrotx(x*cos(radians(Rippleszangle)) + y*sin(radians(Rippleszangle)), -x*sin(radians(Rippleszangle)) + y*cos(radians(Rippleszangle)), z) }
#local Wavesrotz=function {Wavesrotx(x*cos(radians(Waveszangle)) + y*sin(radians(Waveszangle)), -x*sin(radians(Waveszangle)) + y*cos(radians(Waveszangle)), z) }
#local Dentsrotz=function {Dentsrotx(x*cos(radians(Dentszangle)) + y*sin(radians(Dentszangle)), -x*sin(radians(Dentszangle)) + y*cos(radians(Dentszangle)), z) }
#local Cracklerotz=function {Cracklerotx(x*cos(radians(Cracklezangle)) + y*sin(radians(Cracklezangle)), -x*sin(radians(Cracklezangle)) + y*cos(radians(Cracklezangle)), z) }

//------------------------
// (5) finally, rotation around y:
#local Wrinklesroty=function {Wrinklesrotz(x*cos(radians(Wrinklesyangle)) - z*sin(radians(Wrinklesyangle)), y, x*sin(radians(Wrinklesyangle)) + z*cos(radians(Wrinklesyangle))) }
#local Ripplesroty=function {Ripplesrotz(x*cos(radians(Ripplesyangle)) - z*sin(radians(Ripplesyangle)), y, x*sin(radians(Ripplesyangle)) + z*cos(radians(Ripplesyangle))) }
#local Wavesroty=function {Wavesrotz(x*cos(radians(Wavesyangle)) - z*sin(radians(Wavesyangle)), y, x*sin(radians(Wavesyangle)) + z*cos(radians(Wavesyangle))) }
#local Dentsroty=function {Dentsrotz(x*cos(radians(Dentsyangle)) - z*sin(radians(Dentsyangle)), y, x*sin(radians(Dentsyangle)) + z*cos(radians(Dentsyangle))) }
#local Crackleroty=function {Cracklerotz(x*cos(radians(Crackleyangle)) - z*sin(radians(Crackleyangle)), y, x*sin(radians(Crackleyangle)) + z*cos(radians(Crackleyangle))) }

//------------------------
// (6) Landscape pattern:
#local P=function {  Hetero(x, y, z)
                     #if (Noised)
										 * f_noise3d(x, y, z)*Noise
										 #end
										 #if (Ridged)
										 * Ridge(x, y, z)
										 #end
										 #if (Wrinkled)
                       + Wrinklesroty(x, y, z)*Wrinklesgain
                     #end
                     #if (Rippled)
                       - Ripplesroty(x, y, z)*Ripplesgain
                     #end
                     #if (Waved)
                       + Wavesroty(x, y, z)*Wavesgain
                     #end
                     #if (Dented)
										   + Dentsroty(x, y, z)*Dentsgain
										 #end
                     #if (Crackled)
                       * Crackleroty(x, 0, z)*Cracklegain
                     #end
										 #if (Sphered)
                       - Sphere(x+y*tan(radians(SphereShearX)), y, z+y*tan(radians(SphereShearZ)))*Spheregain
                     #end
                    }


//------------------------
// (7) Isosurface:
#declare Landscape =
isosurface {
  function {y - P(x, y, z)}
  //open  
  max_gradient MaxGrad
  accuracy 0.001
  max_trace 1
  contained_by {box {<0.5-Xsize*0.75, LowerBound, 0.5-Zsize*0.75>, <0.5+Xsize*0.75, UpperBound, 0.5+Zsize*0.75>}}    
  translate <-0.5, 0, -0.5>
  texture {Landtex}
  scale Isoscale
  translate Isotrans
}

#end // end of the Geomorph macro

//======================================================================================================================================

#if (Standalone)

// Before calling the macro, use or change any of these parameters (defaults are defined inside the macro).
// (0) Declared landscape parameters:
// patterns:
#declare Noised = false;
#declare Ridged = false;
#declare Wrinkled = true;
#declare Rippled = false;
#declare Waved = false;
#declare Dented = false;
#declare Sphered = true;									// when true, generates an island or an isolated rock
#declare Crackled = false;

// warp:
#declare Wrinkleswarp = true;
#declare Rippleswarp = false;
#declare Waveswarp = false;
#declare Dentswarp = true;
#declare Cracklewarp = false;

// gain:
#declare Wrinklesgain = 0.5;  
#declare Ripplesgain = 0.5;  
#declare Wavesgain = 0.5;  
#declare Dentsgain = 0.5; 
#declare Spheregain = 0.5;
#declare Cracklegain = 1.0; 

// warp turbulence:
#declare Wrinklesturb = <0.5, 0.9, 0.9>;
#declare Ripplesturb = <0.5, 0.9, 0.9>;
#declare Wavesturb = <0.5, 0.9, 0.9>;
#declare Dentsturb = <0.5, 0.9, 0.9>;
#declare Crackleturb = <0.5, 0.9, 0.9>;

// scale:
#declare Wrinklescale = <2, 0.5, 1>;
#declare Ripplescale = <0.5, 0.2, 0.5>;
#declare Wavescale = <1.0, 0.5, 1.0>;
#declare Dentscale = <0.5, 0.2, 0.5>;
#declare Cracklescale = <0.5, 0.2, 0.5>;
#declare Cracksize = 1;

// angle:
#declare Wrinklesangle = <0, 0, -30>;
#declare Ripplesangle = <0, 0, -25>;
#declare Wavesangle = <0, 0, 0>;
#declare Dentsangle = <0, 0, -10>;
#declare Crackleangle = <0, 0, 0>;

// sphere:
#declare SphereShearX = -30;
#declare SphereShearZ = 0;
#declare Diameter = 1;

// noise and randomness:
#declare Noise = 0.4;
#declare Stream = 32;

// isosurface:
#declare UpperBound = 2;								// Bounding box upper limit
#declare LowerBound = -2;								// Bounding box lower limit
#declare MaxGrad = 11;									// max_gradient of the isosurface (change according to need)
#declare Xsize = 15;										// increase size of landscape (not scale!) for X (i.e. the bounding box)
#declare Zsize = 15;										// increase size of landscape (not scale!) for Z (i.e. the bounding box)
#declare Isoscale = <3, 3, 3>;
#declare Isotrans = <0, 0, 0>;

// texture:
//#declare Landtex =										// define your own texture here
// for the internal textures, use:
#declare Simpletex = false;							// just a choice between two nice textures
#declare Slope = true;									// true = slope & altitude; false = gradient texture
#declare SlopeValue = 0.6;							// (0-1) higher values: more slope, less altitude, and vice versa


// now call the macro...:
Geomorph()

// ...and the landscape:
object {Landscape}
// remember that you can use this object for the trace function

//=====================================================================================================================================
camera {
  location  <0, 5, -30> 
  look_at   <0, 0, 0> 
  right     x*image_width/image_height
  angle     54
}

sky_sphere {
  pigment {
    function {abs(y)}
    color_map {[0.0 color blue 0.9] [1.0 color rgb 1]}
  }
}

light_source {<-1000,2000,-1000> colour rgb 1*2}
light_source {<1000,1000,-1000> colour rgb 0.8 shadowless}

fog {fog_type 1 distance 150 color rgb <0.5, 0.5, 0.7>}


#end // end of Standalone

